package com.indicee.api.connector;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLEncoder;
import java.util.Map;

import oauth.signpost.basic.DefaultOAuthConsumer;
import oauth.signpost.commonshttp.CommonsHttpOAuthConsumer;

import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.conn.ssl.SSLSocketFactory;
import org.apache.http.entity.StringEntity;
import org.apache.http.entity.mime.MultipartEntity;
import org.apache.http.entity.mime.content.ContentBody;
import org.apache.http.entity.mime.content.InputStreamBody;
import org.apache.http.entity.mime.content.StringBody;
import org.apache.http.impl.client.DefaultHttpClient;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;

import com.indicee.api.connector.exception.IndiceeAPIConnectorException;
import com.indicee.api.connector.exception.IndiceeAPIStatusException;
import com.indicee.api.connector.util.InputStreamWrapper;
import com.indicee.api.services.ContributionService;
import com.indicee.api.services.IndiceeAPIService;
import com.indicee.api.services.ReportService;
import com.indicee.api.services.impl.ReportServiceImpl;
import com.sun.org.apache.xerces.internal.parsers.DOMParser;

/**
 * The IndiceeAPIConnector provides methods to make API calls to Indicee. There
 * are methods to make both GET and POST calls. The Connector can either be used
 * directly or used with the {@link IndiceeAPIService}s. For example:
 * <p>
 * 
 * <pre>
 * IndiceeAPIConnector connector = new IndiceeAPIConnector(host, consumerKey, consumerSecret, tokenKey, tokenSecret);
 * {@link ReportService} reportService = new {@link ReportServiceImpl}(connector);
 * </pre>
 * 
 * @author victork
 * 
 */
public class IndiceeAPIConnector {

    /**
     * This will convert an input stream to a string.
     * 
     * @param is
     * @return
     */
    private static String convertStreamToString(InputStream is) {
        BufferedReader reader = new BufferedReader(new InputStreamReader(is));
        StringBuilder sb = new StringBuilder();
        String line = null;
        try {
            while ((line = reader.readLine()) != null) {
                sb.append(line + "\n");
            }

            is.close();
        } catch (IOException e) {
            throw new RuntimeException("Unexpected error converting returned body content to String.");
        }
        return sb.toString();
    }

    private final String host;
    @SuppressWarnings("unused")
    private final String tokenKey;
    @SuppressWarnings("unused")
    private final String tokenSecret;
    @SuppressWarnings("unused")
    private final String consumerKey;
    @SuppressWarnings("unused")
    private final String consumerSecret;

    //Warning this is not thread safe.
    private final CommonsHttpOAuthConsumer commonsHttpConsumer;

    /**
     * Creates a new Indicee connector. You can use this class to directly make
     * calls to the Indicee API.
     * <p>
     * The IndiceeAPIConnector is also used when constructing
     * {@link IndiceeAPIService}, such as the {@link ReportService}, or the
     * {@link ContributionService}.
     * 
     * 
     * @param host
     *            The Indicee server to connect to. For example
     *            "https://secure.indicee.com"
     * @param consumerKey
     *            The consumer key
     * @param consumerSecret
     *            The consumer secret
     * @param tokenKey
     *            The toekn key
     * @param tokenSecret
     *            The token secret
     * 
     * @see IndiceeAPIService
     * @throws IllegalArgumentException
     *             If any of the parameters are null or empty. Also if the host
     *             is not a valid url.
     */
    public IndiceeAPIConnector(String host, String consumerKey, String consumerSecret, String tokenKey,
            String tokenSecret) {
        //validate the arguements
        {
            if (host == null || host.isEmpty()) {
                throw new IllegalArgumentException("The host cannot be null or empty.");
            } else if (consumerKey == null || consumerKey.isEmpty()) {
                throw new IllegalArgumentException("The consumerKey cannot be null or empty.");
            } else if (consumerSecret == null || consumerSecret.isEmpty()) {
                throw new IllegalArgumentException("The consumerSecret cannot be null or empty.");
            } else if (tokenKey == null || tokenKey.isEmpty()) {
                throw new IllegalArgumentException("The tokenKey cannot be null or empty.");
            } else if (tokenSecret == null || tokenSecret.isEmpty()) {
                throw new IllegalArgumentException("The tokenSecret cannot be null or empty.");
            }
            //Test to see if the host url is a valid url.
            try {
                new URL(host);
            } catch (MalformedURLException e) {
                throw new IllegalArgumentException("The host url is invalid: " + e.getMessage());
            }
        }
        this.host = host;
        this.tokenKey = tokenKey;
        this.tokenSecret = tokenSecret;
        this.consumerKey = consumerKey;
        this.consumerSecret = consumerSecret;

        // oauth initialization
        DefaultOAuthConsumer oconsumer = new DefaultOAuthConsumer(consumerKey, consumerSecret);
        oconsumer.setTokenWithSecret(tokenKey, tokenSecret);

        commonsHttpConsumer = new CommonsHttpOAuthConsumer(oconsumer.getConsumerKey(), oconsumer.getConsumerSecret());
        commonsHttpConsumer.setTokenWithSecret(oconsumer.getToken(), oconsumer.getTokenSecret());
    }

    private InputStream makeCallInternal(HttpUriRequest request) throws IndiceeAPIConnectorException {
        // init a http client
        DefaultHttpClient httpClient = new DefaultHttpClient();
        SSLSocketFactory sf = (SSLSocketFactory) httpClient.getConnectionManager().getSchemeRegistry().getScheme(
                "https").getSocketFactory();
        sf.setHostnameVerifier(SSLSocketFactory.ALLOW_ALL_HOSTNAME_VERIFIER);

        signRequest(request);

        // send the data to get the response
        HttpResponse httpResponse;
        try {
            httpResponse = httpClient.execute(request);
        } catch (Exception e) {
            throw new IndiceeAPIConnectorException("Error sending HTTP request.", request.getURI().getPath(), e);
        }

        int status = httpResponse.getStatusLine().getStatusCode();
        InputStream body;
        try {
            body = httpResponse.getEntity().getContent();
        } catch (Exception e) {
            throw new IndiceeAPIConnectorException("Unexpected error getting response body.", request.getURI()
                    .getPath(), e);
        }
        //Checks to see if the call completed successfully
        if (status != 200) {
            String rtn = convertStreamToString(body);
            throw new IndiceeAPIStatusException(status, rtn, request.getURI().getPath());
        }
        return body;
    }

    /**
     * This is needed because the commonsHttpConsumer is not thread safe.
     * 
     * @param request
     * @throws IndiceeAPIConnectorException
     */
    private synchronized void signRequest(HttpUriRequest request) throws IndiceeAPIConnectorException {
        try {
            // authenticate the request with oauth
            commonsHttpConsumer.sign(request);
        } catch (Exception e) {
            throw new IndiceeAPIConnectorException("Error signing HTTP request with OAuth.",
                    request.getURI().getPath(), e);
        }
    }

    /**
     * Makes a GET call to indicee.
     * 
     * @param path
     *            The url to connect to.
     * @param parameters
     *            The parameters to send. If null no parameters will be sent.
     * @return An Input stream of the response from the Indicee server.
     * @throws IndiceeAPIConnectorException
     */
    private InputStream makeGetCallInternal(String path) throws IndiceeAPIConnectorException {
        HttpGet httpGet = new HttpGet(host + path);
        return makeCallInternal(httpGet);
    }

    /**
     * Makes a GET call to the Indicee API, on the given path. This method will
     * return a parsed xml {@link Document}.
     * 
     * @param path
     *            The path to connect to.
     * @return A parsed xml {@link Document} of the server response.
     * @throws IndiceeAPIConnectorException
     *             Thrown if there was an error connecting to the server, or if
     *             the server returned an error.
     */
    public Document makeGetCallRtnDocument(String path) throws IndiceeAPIConnectorException {
        return makeGetCallRtnDocument(path, null);
    }

    /**
     * Makes a GET call to the Indicee API, on the given path with the given
     * parameters. This method will return a parsed xml {@link Document}.
     * 
     * @param path
     *            The path to connect to.
     * @param parameters
     *            The parameters to send to the server. If null no parameters
     *            will be sent.
     * @return A parsed xml {@link Document} of the server response.
     * @throws IndiceeAPIConnectorException
     *             Thrown if there was an error connecting to the server, or if
     *             the server returned an error.
     */
    public Document makeGetCallRtnDocument(String path, Map<String, ? extends Object> parameters)
            throws IndiceeAPIConnectorException {
        InputStream body = makeGetCallInternal(path);
        //parse the returned xml
        DOMParser parser = new DOMParser();
        try {
            parser.parse(new InputSource(body));
        } catch (Exception e) {
            throw new RuntimeException("Unexpected Exception parsing xml returned by server.", e);
        }
        return parser.getDocument();
    }

    /**
     * Makes a GET call to the Indicee API, on the given path. This method will
     * return a String.
     * 
     * @param path
     *            The path to connect to.
     * @return A String of the server response.
     * @throws IndiceeAPIConnectorException
     *             Thrown if there was an error connecting to the server, or if
     *             the server returned an error.
     */
    public String makeGetCallRtnString(String path) throws IndiceeAPIConnectorException {
        return convertStreamToString(makeGetCallInternal(path));
    }

    /**
     * Makes a POST call to indicee.
     * 
     * @param path
     *            The url to connect to.
     * @param parameters
     *            The parameters to send. If null no parameters will be sent.
     * @return An Input stream of the response from the Indicee server.
     * @throws IndiceeAPIConnectorException
     */
    @SuppressWarnings("deprecation")
    private InputStream makePostCallInternal(String path, Map<String, ? extends Object> parameters)
            throws IndiceeAPIConnectorException {
        HttpPost httpPost = new HttpPost(host + path);
        if (parameters != null && !parameters.isEmpty()) {

            if (parametersContainsFileOrInputStream(parameters)) {

                // build post data. File upload needs multipart form data.
                // see http://www.w3.org/TR/html401/interact/forms.html#h-17.13.4.2
                MultipartEntity mentity = new MultipartEntity();

                for (String key : parameters.keySet()) {
                    Object value = parameters.get(key);
                    if (value instanceof File) {
                        ContentBody fbody;
                        try {
                            fbody = new InputStreamBody(new FileInputStream((File) value), ((File) value).getName());
                        } catch (FileNotFoundException e) {
                            throw new IndiceeAPIConnectorException("Error sending file: " + ((File) value).getName()
                                    + " could not find file on local machine.", path, e);
                        }
                        mentity.addPart(key, fbody);
                    } else if (value instanceof InputStreamWrapper) {
                        ContentBody fbody;
                        InputStreamWrapper streamWrapper = (InputStreamWrapper) value;
                        fbody = new InputStreamBody(streamWrapper.getInputSteam(), streamWrapper.getMimeType()
                                .getContentType(), streamWrapper.getFileName());
                        mentity.addPart(key, fbody);
                    } else {
                        try {
                            mentity.addPart(key, new StringBody(parameters.get(key).toString()));
                        } catch (UnsupportedEncodingException e) {
                            throw new IndiceeAPIConnectorException("Unexpected error encoding string.", path, e);
                        }
                    }
                }
                httpPost.setEntity(mentity);
            } else {
                String body = "";
                for (String key : parameters.keySet()) {
                    body += key + "=" + URLEncoder.encode(parameters.get(key).toString()) + "&";
                }
                body = body.substring(0, body.length() - 1);
                StringEntity entity;
                try {
                    entity = new StringEntity(body);
                } catch (UnsupportedEncodingException e) {
                    throw new IndiceeAPIConnectorException("Unexpected error encoding string.", path, e);
                }
                entity.setContentType("application/x-www-form-urlencoded");
                httpPost.setEntity(entity);
            }
        }

        return makeCallInternal(httpPost);
    }

    /**
     * Makes a POST call to the Indicee API, on the given path. This method will
     * return a parsed xml {@link Document}.
     * 
     * @param path
     *            The path to connect to.
     * @return A parsed xml {@link Document} of the server response.
     * @throws IndiceeAPIConnectorException
     *             Thrown if there was an error connecting to the server, or if
     *             the server returned an error.
     */
    public Document makePostCallRtnDocument(String path) throws IndiceeAPIConnectorException {
        return makePostCallRtnDocument(path, null);
    }

    /**
     * Makes a POST call to the Indicee API, on the given path with the given
     * parameters. This method will return a parsed xml {@link Document}.
     * 
     * @param path
     *            The path to connect to.
     * @param parameters
     *            The parameters to send to the server. If null no parameters
     *            will be sent.
     * @return A parsed xml {@link Document} of the server response.
     * @throws IndiceeAPIConnectorException
     *             Thrown if there was an error connecting to the server, or if
     *             the server returned an error.
     */
    public Document makePostCallRtnDocument(String path, Map<String, ? extends Object> parameters)
            throws IndiceeAPIConnectorException {
        InputStream body = makePostCallInternal(path, parameters);
        DOMParser parser = new DOMParser();
        try {
            parser.parse(new InputSource(body));
        } catch (Exception e) {
            throw new RuntimeException("Unexpected Exception parsing xml returned by server.", e);
        }
        return parser.getDocument();
    }

    /**
     * Makes a POST call to the Indicee API, on the given path. This method will
     * return a String.
     * 
     * @param path
     *            The path to connect to.
     * @return A String of the server response.
     * @throws IndiceeAPIConnectorException
     *             Thrown if there was an error connecting to the server, or if
     *             the server returned an error.
     */
    public String makePostCallRtnString(String path) throws IndiceeAPIConnectorException {
        return makePostCallRtnString(path, null);
    }

    /**
     * Makes a POST call to the Indicee API, on the given path with the given
     * parameters. This method will return a String.
     * 
     * @param path
     *            The path to connect to.
     * @param parameters
     *            The parameters to send to the server. If null no parameters
     *            will be sent.
     * @return A String of the server response.
     * @throws IndiceeAPIConnectorException
     *             Thrown if there was an error connecting to the server, or if
     *             the server returned an error.
     */
    public String makePostCallRtnString(String path, Map<String, Object> parameters)
            throws IndiceeAPIConnectorException {
        return convertStreamToString(makePostCallInternal(path, parameters));
    }

    private boolean parametersContainsFileOrInputStream(Map<String, ? extends Object> parameters) {
        for (Object value : parameters.values()) {
            if (value instanceof File || value instanceof InputStreamWrapper) {
                return true;
            }
        }
        return false;
    }
}
