package com.indicee.api.objects;

import java.io.Serializable;

/**
 * This is a view window to be used when using the api to return reports data.
 * 
 * @author victork
 * 
 */
public class ViewWindow implements Serializable {

    private static final long serialVersionUID = 3231037015409098097L;

    private final int rowStart;
    private final int numRows;
    private final int columnStart;
    private final int numColumns;

    /**
     * Creates a default view window that returns the first 100 rows and all
     * columns
     */
    public ViewWindow() {
        this(0, 100);
    }

    /**
     * Creates a view window with the specific row start and num rows.
     * 
     * @param rowStart
     *            The row to start the window at.
     * @param numRows
     *            The number of rows to return.
     */
    public ViewWindow(int rowStart, int numRows) {
        this(rowStart, numRows, 0, Integer.MAX_VALUE);
    }

    /**
     * Creates a view window with all bounds specified.
     * 
     * @param rowStart
     *            The row to start the window at.
     * @param numRows
     *            The number of rows to return.
     * @param columnStart
     *            The column to start the window at.
     * @param numColumns
     *            The number of columns to return.
     */
    public ViewWindow(int rowStart, int numRows, int columnStart, int numColumns) {
        this.rowStart = rowStart;
        this.numRows = numRows;
        this.columnStart = columnStart;
        this.numColumns = numColumns;
    }

    /**
     * One query window is equal to another if all thier bounds are equal.
     */
    @Override
    public boolean equals(Object obj) {
        if (obj != null && obj instanceof ViewWindow) {
            ViewWindow other = (ViewWindow) obj;
            if (rowStart == other.rowStart && numRows == other.numRows && columnStart == other.columnStart
                    && numColumns == other.numColumns) {
                return true;
            }
        }
        return false;
    }

    /**
     * Returns the start column in the view window
     * 
     * @return
     */
    public int getColumnStart() {
        return columnStart;
    }

    /**
     * Returns the number of columns in the view window
     * 
     * @return
     */
    public int getNumColumns() {
        return numColumns;
    }

    /**
     * Returns the number of rows in the view window
     * 
     * @return
     */
    public int getNumRows() {
        return numRows;
    }

    /**
     * Returns the start row in the view window
     * 
     * @return
     */
    public int getRowStart() {
        return rowStart;
    }

    /**
     * Returns a string representation of the view window. This is the actual
     * query parameter.
     */
    @Override
    public String toString() {
        return rowStart + "," + columnStart + "," + numRows + "," + numColumns;
    }
}
