package com.indicee.api.parser;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;

import com.indicee.api.objects.report.Collection;
import com.indicee.api.objects.report.Metadata;
import com.indicee.api.objects.report.Report;
import com.indicee.api.objects.report.VisualizationOptions;
import com.indicee.api.objects.report.data.Data;
import com.indicee.api.objects.report.data.DataRow;
import com.indicee.api.objects.report.view.View;
import com.indicee.api.objects.report.view.attributes.Attribute;
import com.indicee.api.objects.report.view.attributes.AttributesOrigin;
import com.indicee.api.objects.report.view.axes.Axis;
import com.indicee.api.objects.report.view.axes.AxisSort;
import com.indicee.api.objects.report.view.axes.GroupFilter;
import com.indicee.api.objects.report.view.axes.GroupingAxisLayer;
import com.indicee.api.objects.report.view.axes.Layer;
import com.indicee.api.objects.report.view.axes.LayerGroupSorting;
import com.indicee.api.objects.report.view.axes.LayerQueryGrouping;
import com.indicee.api.objects.report.view.axes.QueryValue;
import com.indicee.api.objects.report.view.axes.ValuesAxislayer;
import com.indicee.api.objects.report.view.filter.FieldFilter;
import com.indicee.api.objects.report.view.filter.FieldFilterValue;
import com.indicee.api.objects.report.view.filter.Filter;
import com.indicee.api.objects.report.view.filter.RelativeTimePeriod;
import com.indicee.api.objects.report.view.filter.RelativeTimePeriodFilter;

public final class ReportParser extends IndiceeParser {

    private enum MetadataElement {
        Measures("measures"), Groups("groups");

        public static MetadataElement get(String name) {
            for (MetadataElement item : MetadataElement.values()) {
                if (item.value.equals(name)) {
                    return item;
                }
            }
            return null;
        }

        private final String value;

        private MetadataElement(String value) {
            this.value = value;
        }
    }

    private enum ReportAttributesElement {
        Origin("origin"), Attribute("attribute");

        public static ReportAttributesElement get(String name) {
            for (ReportAttributesElement item : ReportAttributesElement.values()) {
                if (item.value.equals(name)) {
                    return item;
                }
            }
            return null;
        }

        private final String value;

        private ReportAttributesElement(String value) {
            this.value = value;
        }
    }

    private enum ReportDataElement {
        Row("row"), Cell("cell");

        public static ReportDataElement get(String name) {
            for (ReportDataElement item : ReportDataElement.values()) {
                if (item.value.equals(name)) {
                    return item;
                }
            }
            return null;
        }

        private final String value;

        private ReportDataElement(String value) {
            this.value = value;
        }
    }

    private enum ReportElement {
        ID("id"), Name("name"), OwnerID("owner_id"), Deleted("deleted"), CreatedDateTime("created_datetime"), Metadata(
                "metadata"), Visualization("vis"), View("view"), Data("data"), Report("report");

        public static ReportElement get(String name) {
            for (ReportElement item : ReportElement.values()) {
                if (item.value.equals(name)) {
                    return item;
                }
            }
            return null;
        }

        private final String value;

        private ReportElement(String value) {
            this.value = value;
        }
    }

    private enum ReportFieldFilterElement {
        FieldFilter("fieldfilter"), RelativeTimePeriodFilter("relativetimeperiodfilter"), RelativeTimePeriods(
                "relativetimeperiods"), RelativeTimePeriod("relativetimeperiod"), FieldFilterValues("values"), FieldFilterValue(
                "value");

        public static ReportFieldFilterElement get(String name) {
            for (ReportFieldFilterElement item : ReportFieldFilterElement.values()) {
                if (item.value.equals(name)) {
                    return item;
                }
            }
            return null;
        }

        private final String value;

        private ReportFieldFilterElement(String value) {
            this.value = value;
        }
    }

    private enum ReportViewAxisElement {
        Axis("axis"), Layers("layers"), AxisSorting("axissorting"), GroupingAxisLayer("groupingaxislayer"), QueryGrouping(
                "querygrouping"), GroupSorting("groupsorting"), Properties("properties"), QueryValue("queryvalue"), ValuesAxisLayer(
                "valuesaxislayer"), Values("values"), Attributes("attributes"), AxisSort("axissort"), GroupFilters(
                "groupfilters"), GroupFilter("groupfilter"), ComparatorAttributes("comparatorattributes");

        public static ReportViewAxisElement get(String name) {
            for (ReportViewAxisElement item : ReportViewAxisElement.values()) {
                if (item.value.equals(name)) {
                    return item;
                }
            }
            return null;
        }

        private final String value;

        private ReportViewAxisElement(String value) {
            this.value = value;
        }
    }

    private enum ReportViewElement {
        Axes("axes"), FieldFilters("fieldfilters"), Collections("collections"), Attributes("attributes");

        public static ReportViewElement get(String name) {
            for (ReportViewElement item : ReportViewElement.values()) {
                if (item.value.equals(name)) {
                    return item;
                }
            }
            return null;
        }

        private final String value;

        private ReportViewElement(String value) {
            this.value = value;
        }
    }

    public static final SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'");

    public static Report createReport(Document reportDocument) {
        return getReport(reportDocument.getDocumentElement());
    }

    public static Report createReport(String reportDocument) {
        return createReport(getDocumentFromString(reportDocument));
    }

    public static List<Report> createReports(Document reportsDocument) {
        List<Report> reports = new ArrayList<Report>();
        for (int i = 0; i < reportsDocument.getDocumentElement().getChildNodes().getLength(); i++) {
            Node child = reportsDocument.getDocumentElement().getChildNodes().item(i);
            ReportElement elem = ReportElement.get(child.getNodeName());
            if (elem != null && elem.equals(ReportElement.Report)) {
                reports.add(getReport(child));
            }
        }
        return reports;
    }

    public static List<Report> createReports(String reportsDocument) {
        return createReports(getDocumentFromString(reportsDocument));
    }

    public static View createReportView(Document reportViewDocument) {
        return getView(reportViewDocument.getElementsByTagName(ReportElement.View.value).item(0));
    }

    public static View createReportView(String reportViewDocument) {
        return createReportView(getDocumentFromString(reportViewDocument));
    }

    private static List<AttributesOrigin> getOriginAttributes(Node attribtutesNode) {
        List<AttributesOrigin> originAttributes = new ArrayList<AttributesOrigin>();
        for (int i = 0; i < attribtutesNode.getChildNodes().getLength(); i++) {
            Node child = attribtutesNode.getChildNodes().item(i);
            ReportAttributesElement elem = ReportAttributesElement.get(child.getNodeName());
            if (elem != null && elem.equals(ReportAttributesElement.Origin)) {
                originAttributes.add(getOriginAttribute(child));
            }
        }
        return originAttributes;
    }

    private static List<Axis> getAxes(Node axesNode) {
        List<Axis> axes = new ArrayList<Axis>();
        for (int i = 0; i < axesNode.getChildNodes().getLength(); i++) {
            Node axisNode = axesNode.getChildNodes().item(i);
            ReportViewAxisElement elem = ReportViewAxisElement.get(axisNode.getNodeName());
            if (elem != null && elem.equals(ReportViewAxisElement.Axis)) {
                Axis axis = new Axis();
                NamedNodeMap entryAttributes = axisNode.getAttributes();
                axis.setName(entryAttributes.getNamedItem("name").getNodeValue());

                for (int j = 0; j < axisNode.getChildNodes().getLength(); j++) {
                    Node child = axisNode.getChildNodes().item(j);
                    elem = ReportViewAxisElement.get(child.getNodeName());
                    if (elem != null) {
                        switch (elem) {
                        case Layers:
                            axis.setLayers(getLayers(child));
                            break;
                        case AxisSorting:
                            axis.setAxisSortings(getAxisSortings(child));
                            break;
                        }
                    }
                }

                axes.add(axis);
            }
        }
        return axes;
    }

    private static List<AxisSort> getAxisSortings(Node axisSortingsNode) {
        List<AxisSort> axisSortings = new ArrayList<AxisSort>();
        for (int i = 0; i < axisSortingsNode.getChildNodes().getLength(); i++) {
            Node axisSortNode = axisSortingsNode.getChildNodes().item(i);
            ReportViewAxisElement elem = ReportViewAxisElement.get(axisSortNode.getNodeName());
            if (elem != null && elem.equals(ReportViewAxisElement.AxisSort)) {
                AxisSort axisSort = new AxisSort();
                NamedNodeMap entryAttributes = axisSortNode.getAttributes();
                axisSort.setAscending(Boolean.parseBoolean(entryAttributes.getNamedItem("ascending").getNodeValue()));
                axisSort.setBreakHierarchies(Boolean.parseBoolean(entryAttributes.getNamedItem("breakhierarchies")
                        .getNodeValue()));

                for (int j = 0; j < axisSortNode.getChildNodes().getLength(); j++) {
                    Node child = axisSortNode.getChildNodes().item(j);
                    elem = ReportViewAxisElement.get(child.getNodeName());
                    if (elem != null) {
                        switch (elem) {
                        case QueryValue:
                            axisSort.setQueryValue(getQueryValue(child));
                            break;
                        case Attributes:
                            axisSort.setAttributes(getOriginAttributes(child));
                            break;
                        }
                    }
                }

                axisSortings.add(axisSort);
            }
        }
        return axisSortings;
    }

    private static List<Collection> getCollections(Node child) {
        // TODO Auto-generated method stub
        return null;
    }

    private static Date getCreatedDateTime(Node child) {
        try {
            return dateFormat.parse(child.getFirstChild().getNodeValue());
        } catch (Exception e) {
            throw new RuntimeException("Unexpected exception parsing date.", e);
        }
    }

    private static Data getData(Node dataNode) {
        Data reportData = new Data();
        reportData.setTotalColumns(Integer.parseInt(dataNode.getAttributes().getNamedItem("cols").getNodeValue()));
        reportData.setTotalRows(Integer.parseInt(dataNode.getAttributes().getNamedItem("rows").getNodeValue()));
        List<DataRow> rows = new ArrayList<DataRow>();
        for (int i = 0; i < dataNode.getChildNodes().getLength(); i++) {
            DataRow reportDataRow = new DataRow();
            if (dataNode.getChildNodes().item(i).getChildNodes().getLength() > 0) {
                List<String> cells = new ArrayList<String>();
                for (int j = 0; j < dataNode.getChildNodes().item(i).getChildNodes().getLength(); j++) {
                    Node rowCellNode = dataNode.getChildNodes().item(i).getChildNodes().item(j);
                    if (rowCellNode != null && ReportDataElement.Cell.value.equals(rowCellNode.getLocalName())) {
                        if (rowCellNode.hasChildNodes()) {
                            cells.add(rowCellNode.getFirstChild().getNodeValue());
                        } else {
                            cells.add(null);
                        }
                    }
                }
                reportDataRow.setCells(cells);
                rows.add(reportDataRow);
            }
        }
        reportData.setReportDataRows(rows);
        return reportData;
    }

    private static FieldFilter getFieldFilter(Node fieldFilterNode) {
        FieldFilter fieldFilter = new FieldFilter();
        NamedNodeMap entryAttributes = fieldFilterNode.getAttributes();
        fieldFilter.setField(entryAttributes.getNamedItem("field").getNodeValue());
        fieldFilter.setInclusive(Boolean.parseBoolean(entryAttributes.getNamedItem("inclusive").getNodeValue()));
        if (entryAttributes.getNamedItem("entity") != null) {
            fieldFilter.setEntity(entryAttributes.getNamedItem("entity").getNodeValue());
        }
        if (entryAttributes.getNamedItem("value") != null) {
            List<FieldFilterValue> values = new ArrayList<FieldFilterValue>();
            FieldFilterValue value = new FieldFilterValue();
            value.setValue(entryAttributes.getNamedItem("value").getNodeValue());
            values.add(value);
            fieldFilter.setValues(values);
        } else {
            for (int i = 0; i < fieldFilterNode.getChildNodes().getLength(); i++) {
                Node fieldFilterValuesNode = fieldFilterNode.getChildNodes().item(i);
                ReportFieldFilterElement elem = ReportFieldFilterElement.get(fieldFilterValuesNode.getNodeName());
                if (elem != null && elem.equals(ReportFieldFilterElement.FieldFilterValues)) {
                    List<FieldFilterValue> values = new ArrayList<FieldFilterValue>();
                    for (int j = 0; j < fieldFilterValuesNode.getChildNodes().getLength(); j++) {
                        Node fieldFilterValueNode = fieldFilterValuesNode.getChildNodes().item(j);
                        elem = ReportFieldFilterElement.get(fieldFilterValueNode.getNodeName());
                        if (elem != null && elem.equals(ReportFieldFilterElement.FieldFilterValue)) {
                            FieldFilterValue value = new FieldFilterValue();
                            entryAttributes = fieldFilterValueNode.getAttributes();
                            value.setValue(entryAttributes.getNamedItem("value").getNodeValue());
                            values.add(value);
                        }
                    }
                    fieldFilter.setValues(values);
                    break;
                }
            }
        }
        return fieldFilter;
    }

    private static List<Filter> getFieldFilters(Node fieldFiltersNode) {
        List<Filter> fieldFilters = new ArrayList<Filter>();
        for (int i = 0; i < fieldFiltersNode.getChildNodes().getLength(); i++) {
            Node child = fieldFiltersNode.getChildNodes().item(i);
            ReportFieldFilterElement elem = ReportFieldFilterElement.get(child.getNodeName());
            if (elem != null) {
                switch (elem) {
                case FieldFilter:
                    fieldFilters.add(getFieldFilter(child));
                    break;
                case RelativeTimePeriodFilter:
                    fieldFilters.add(getRelativeTimePeriodFilter(child));
                    break;
                }
            }
        }
        return fieldFilters;
    }

    private static LayerGroupSorting getGroupSorting(Node groupSortingNode) {
        LayerGroupSorting groupSorting = new LayerGroupSorting();
        NamedNodeMap entryAttributes = groupSortingNode.getAttributes();
        groupSorting.setAscending(Boolean.parseBoolean(entryAttributes.getNamedItem("ascending").getNodeValue()));
        groupSorting.setSortByCaption(Boolean
                .parseBoolean(entryAttributes.getNamedItem("sortbycaption").getNodeValue()));
        return groupSorting;
    }

    private static List<Layer> getLayers(Node layersNode) {
        List<Layer> layers = new ArrayList<Layer>();
        for (int i = 0; i < layersNode.getChildNodes().getLength(); i++) {
            Node layerNode = layersNode.getChildNodes().item(i);
            ReportViewAxisElement elem = ReportViewAxisElement.get(layerNode.getNodeName());
            if (elem != null) {
                switch (elem) {
                case GroupingAxisLayer:
                    GroupingAxisLayer groupingAxisLayer = new GroupingAxisLayer();
                    NamedNodeMap entryAttributes = layerNode.getAttributes();
                    groupingAxisLayer.setLeadingTotals(Boolean.parseBoolean(entryAttributes.getNamedItem(
                            "leadingtotals").getNodeValue()));
                    groupingAxisLayer.setTrailingTotals(Boolean.parseBoolean(entryAttributes.getNamedItem(
                            "trailingtotals").getNodeValue()));
                    groupingAxisLayer.setHierarchy(entryAttributes.getNamedItem("hierarchy").getNodeValue());
                    groupingAxisLayer.setHierarchyDisplay(entryAttributes.getNamedItem("hierarchydisplay")
                            .getNodeValue());
                    groupingAxisLayer.setDrillable(Boolean.parseBoolean(entryAttributes.getNamedItem("drillable")
                            .getNodeValue()));

                    for (int j = 0; j < layerNode.getChildNodes().getLength(); j++) {
                        Node child = layerNode.getChildNodes().item(j);
                        elem = ReportViewAxisElement.get(child.getNodeName());
                        if (elem != null) {
                            switch (elem) {
                            case QueryGrouping:
                                groupingAxisLayer.setQueryGrouping(getQueryGrouping(child));
                                break;
                            case GroupSorting:
                                groupingAxisLayer.setGroupSorting(getGroupSorting(child));
                                break;
                            case GroupFilters:
                                groupingAxisLayer.setGroupFilters(getGroupFilters(child));
                                break;
                            }
                        }
                    }

                    layers.add(groupingAxisLayer);
                    break;
                case ValuesAxisLayer:
                    ValuesAxislayer valuesaxislayer = new ValuesAxislayer();
                    for (int j = 0; j < layerNode.getChildNodes().getLength(); j++) {
                        Node child = layerNode.getChildNodes().item(j);
                        elem = ReportViewAxisElement.get(child.getNodeName());
                        if (elem != null && elem.equals(ReportViewAxisElement.Values)) {
                            valuesaxislayer.setValues(getQueryValues(child));
                        }
                    }
                    layers.add(valuesaxislayer);
                    break;
                }
            }
        }
        return layers;
    }

    private static List<GroupFilter> getGroupFilters(Node groupFiltersNode) {
        List<GroupFilter> groupFilters = new ArrayList<GroupFilter>();
        for (int i = 0; i < groupFiltersNode.getChildNodes().getLength(); i++) {
            Node groupFilterNode = groupFiltersNode.getChildNodes().item(i);
            ReportViewAxisElement elem = ReportViewAxisElement.get(groupFilterNode.getNodeName());
            if (elem != null && elem.equals(ReportViewAxisElement.GroupFilter)) {
                GroupFilter groupFilter = new GroupFilter();
                NamedNodeMap entryAttributes = groupFilterNode.getAttributes();
                groupFilter.setComparator(entryAttributes.getNamedItem("comparator").getNodeValue());

                for (int j = 0; j < groupFilterNode.getChildNodes().getLength(); j++) {
                    Node child = groupFilterNode.getChildNodes().item(j);
                    elem = ReportViewAxisElement.get(child.getNodeName());
                    if (elem != null) {
                        switch (elem) {
                        case QueryValue:
                            groupFilter.setQueryValue(getQueryValue(child));
                            break;
                        case ComparatorAttributes:
                            List<Attribute> attributes = new ArrayList<Attribute>();
                            for (int k = 0; k < child.getChildNodes().getLength(); k++) {
                                Node attributeNode = child.getChildNodes().item(k);
                                ReportAttributesElement attributeElem = ReportAttributesElement.get(attributeNode
                                        .getNodeName());
                                if (attributeElem != null && attributeElem.equals(ReportAttributesElement.Attribute)) {
                                    attributes.add(getAttribute(attributeNode));
                                }
                            }
                            groupFilter.setComparatorAttributes(attributes);
                            break;
                        }
                    }
                }

                groupFilters.add(groupFilter);
            }
        }
        return groupFilters;
    }

    private static Set<Metadata> getMetadata(Node metadataNode) {
        Set<Metadata> metadatas = new HashSet<Metadata>();
        for (int i = 0; i < metadataNode.getChildNodes().getLength(); i++) {
            NamedNodeMap entryAttributes = metadataNode.getChildNodes().item(i).getAttributes();
            if (entryAttributes != null) {
                Metadata reportMetadata = new Metadata(entryAttributes.getNamedItem("key").getNodeValue(),
                        entryAttributes.getNamedItem("value").getNodeValue());
                metadatas.add(reportMetadata);
            }
        }
        return metadatas;
    }

    private static AttributesOrigin getOriginAttribute(Node attribtuteOriginNode) {
        AttributesOrigin originAttribute = new AttributesOrigin();
        NamedNodeMap entryAttributes = attribtuteOriginNode.getAttributes();
        originAttribute.setName(entryAttributes.getNamedItem("name").getNodeValue());

        List<Attribute> reportsAttributes = new ArrayList<Attribute>();
        for (int i = 0; i < attribtuteOriginNode.getChildNodes().getLength(); i++) {
            Node child = attribtuteOriginNode.getChildNodes().item(i);
            ReportAttributesElement elem = ReportAttributesElement.get(child.getNodeName());
            if (elem != null && elem.equals(ReportAttributesElement.Attribute)) {
                reportsAttributes.add(getAttribute(child));
            }
        }
        originAttribute.setReportAttributes(reportsAttributes);
        return originAttribute;
    }

    private static Attribute getAttribute(Node attribtuteNode) {
        Attribute attribute = new Attribute();
        NamedNodeMap entryAttributes = attribtuteNode.getAttributes();
        attribute.setName(entryAttributes.getNamedItem("name").getNodeValue());
        attribute.setType(entryAttributes.getNamedItem("type").getNodeValue());
        attribute.setValue(entryAttributes.getNamedItem("value").getNodeValue());
        return attribute;
    }

    private static LayerQueryGrouping getQueryGrouping(Node queryGroupingNode) {
        LayerQueryGrouping queryGrouping = new LayerQueryGrouping();
        NamedNodeMap entryAttributes = queryGroupingNode.getAttributes();
        queryGrouping.setEntity(entryAttributes.getNamedItem("entity").getNodeValue());

        for (int i = 0; i < queryGroupingNode.getChildNodes().getLength(); i++) {
            Node child = queryGroupingNode.getChildNodes().item(i);
            ReportViewAxisElement elem = ReportViewAxisElement.get(child.getNodeName());
            if (elem != null && elem.equals(ReportViewAxisElement.Properties)) {
                queryGrouping.setProperties(getQueryValues(child));
            }
        }

        return queryGrouping;
    }

    private static List<QueryValue> getQueryValues(Node queryValuesNode) {
        List<QueryValue> queryValues = new ArrayList<QueryValue>();
        for (int i = 0; i < queryValuesNode.getChildNodes().getLength(); i++) {
            Node queryValueNode = queryValuesNode.getChildNodes().item(i);
            ReportViewAxisElement elem = ReportViewAxisElement.get(queryValueNode.getNodeName());
            if (elem != null && elem.equals(ReportViewAxisElement.QueryValue)) {
                queryValues.add(getQueryValue(queryValueNode));
            }
        }
        return queryValues;
    }

    private static QueryValue getQueryValue(Node queryValueNode) {
        QueryValue queryValue = new QueryValue();
        NamedNodeMap entryAttributes = queryValueNode.getAttributes();
        queryValue.setEntity(entryAttributes.getNamedItem("entity").getNodeValue());
        queryValue.setAggregator(entryAttributes.getNamedItem("aggregator").getNodeValue());
        if (entryAttributes.getNamedItem("field") != null) {
            queryValue.setField(entryAttributes.getNamedItem("field").getNodeValue());
        }
        for (int j = 0; j < queryValueNode.getChildNodes().getLength(); j++) {
            Node attributesNode = queryValueNode.getChildNodes().item(j);
            ReportViewAxisElement elem = ReportViewAxisElement.get(attributesNode.getNodeName());
            if (elem != null && elem.equals(ReportViewAxisElement.Attributes)) {
                queryValue.setAttributes(getOriginAttributes(attributesNode));
                break;
            }
        }
        return queryValue;
    }

    private static RelativeTimePeriodFilter getRelativeTimePeriodFilter(Node relativeTimePeriodFilterNode) {
        RelativeTimePeriodFilter relativeTimePeriodFilter = new RelativeTimePeriodFilter();
        NamedNodeMap entryAttributes = relativeTimePeriodFilterNode.getAttributes();
        relativeTimePeriodFilter.setField(entryAttributes.getNamedItem("field").getNodeValue());
        relativeTimePeriodFilter.setInclusive(Boolean.parseBoolean(entryAttributes.getNamedItem("inclusive")
                .getNodeValue()));
        relativeTimePeriodFilter.setCalendar(entryAttributes.getNamedItem("calendar").getNodeValue());

        for (int i = 0; i < relativeTimePeriodFilterNode.getChildNodes().getLength(); i++) {
            Node relativeTimePeriodsNode = relativeTimePeriodFilterNode.getChildNodes().item(i);
            ReportFieldFilterElement elem = ReportFieldFilterElement.get(relativeTimePeriodsNode.getNodeName());
            if (elem != null && elem.equals(ReportFieldFilterElement.RelativeTimePeriods)) {
                List<RelativeTimePeriod> relativeTimePeriods = new ArrayList<RelativeTimePeriod>();
                for (int j = 0; j < relativeTimePeriodsNode.getChildNodes().getLength(); j++) {
                    Node relativeTimePeriodNode = relativeTimePeriodsNode.getChildNodes().item(j);
                    elem = ReportFieldFilterElement.get(relativeTimePeriodNode.getNodeName());
                    if (elem != null && elem.equals(ReportFieldFilterElement.RelativeTimePeriod)) {
                        RelativeTimePeriod relativeTimePeriod = new RelativeTimePeriod();
                        entryAttributes = relativeTimePeriodNode.getAttributes();
                        relativeTimePeriod.setRangeEnd(Integer.parseInt(entryAttributes.getNamedItem("rangeend")
                                .getNodeValue()));
                        relativeTimePeriod.setRangeStart(Integer.parseInt(entryAttributes.getNamedItem("rangestart")
                                .getNodeValue()));
                        relativeTimePeriod.setTimePeriod(entryAttributes.getNamedItem("timeperiod").getNodeValue());
                        if (entryAttributes.getNamedItem("referencetime") != null) {
                            relativeTimePeriod.setReferenceTime(Long.parseLong(entryAttributes.getNamedItem(
                                    "referencetime").getNodeValue()));
                        }
                        relativeTimePeriods.add(relativeTimePeriod);
                    }
                }
                relativeTimePeriodFilter.setRelativeTimePeriods(relativeTimePeriods);
                break;
            }
        }
        return relativeTimePeriodFilter;
    }

    private static Report getReport(Node reportNode) {
        Report report = new Report();
        for (int i = 0; i < reportNode.getChildNodes().getLength(); i++) {
            Node child = reportNode.getChildNodes().item(i);
            ReportElement elem = ReportElement.get(child.getNodeName());
            if (elem != null) {
                switch (elem) {
                case ID:
                    report.setId(getInteger(child));
                    break;
                case Name:
                    report.setName(getString(child));
                    break;
                case OwnerID:
                    report.setOwnerID(getInteger(child));
                    break;
                case Deleted:
                    report.setDeleted(getBoolean(child));
                    break;
                case CreatedDateTime:
                    report.setCreatedDate(getCreatedDateTime(child));
                    break;
                case Metadata:
                    for (int j = 0; j < child.getChildNodes().getLength(); j++) {
                        Node metadataChild = child.getChildNodes().item(j);
                        MetadataElement metadataElem = MetadataElement.get(metadataChild.getNodeName());
                        if (metadataElem != null) {
                            switch (metadataElem) {
                            case Measures:
                                report.setMeasures(getMetadata(metadataChild));
                                break;
                            case Groups:
                                report.setGroups(getMetadata(metadataChild));
                                break;
                            }
                        }
                    }
                    //report.set(getMetadata(child));
                    break;
                case Visualization:
                    report.setVisualizationOptions(getVisualization(child));
                    break;
                case View:
                    report.setReportView(getView(child));
                    break;
                case Data:
                    report.setReportData(getData(child));
                    break;
                }
            }
        }
        return report;
    }

    private static View getView(Node viewNode) {
        View reportView = new View();
        for (int i = 0; i < viewNode.getChildNodes().getLength(); i++) {
            Node child = viewNode.getChildNodes().item(i);
            ReportViewElement elem = ReportViewElement.get(child.getNodeName());
            if (elem != null) {
                switch (elem) {
                case Axes:
                    reportView.setReportViewAxes(getAxes(child));
                    break;
                case FieldFilters:
                    reportView.setFieldFilters(getFieldFilters(child));
                    break;
                case Collections:
                    reportView.setCollections(getCollections(child));
                    break;
                case Attributes:
                    reportView.setAttributeOrigins(getOriginAttributes(child));
                    break;
                }
            }
        }
        return reportView;
    }

    private static VisualizationOptions getVisualization(Node child) {
        // TODO Auto-generated method stub
        return null;
    }
}
