package com.indicee.api.util;

import java.io.File;
import java.io.IOException;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.OptionGroup;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.PosixParser;

import com.indicee.api.connector.IndiceeAPIConnector;
import com.indicee.api.connector.exception.IndiceeAPIConnectorException;
import com.indicee.api.services.ContributionService;
import com.indicee.api.services.impl.ContributionServiceImpl;

/**
 * This is a utility that will uploadfiles to Indicee.
 * <p>
 * Call the using the following syntax:
 * <p>
 * FileUploader -u &lt;url&gt; -ck &lt;consumerKey&gt; -cs
 * &lt;consumerSecret&gt; -tk &lt;tokenKey&gt; -ts &lt;tokenSecret&gt; [-d
 * &lt;uploadDirectory&gt; | -f &lt;uploadFile&gt;]
 * <p>
 * It can either upload an entire directory or a single file.
 * 
 * @author victork
 * 
 */
public class FileUploader {

    private static String host;
    private static String consumerKey;
    private static String consumerSecret;
    private static String tokenKey;
    private static String tokenSecret;
    private static File contributionDirectory;
    private static File contributionFile;

    private static final String URL_OPTION = "u";
    private static final String CONSUMER_KEY_OPTION = "ck";
    private static final String CONSUMER_SECRET_OPTION = "cs";
    private static final String HELP_OPTION = "h";
    private static final String TOKEN_KEY_OPTION = "tk";
    private static final String TOKEN_SECRET_OPTION = "ts";
    private static final String DIRECTORY_OPTION = "d";
    private static final String FILE_OPTION = "f";

    public static void main(String[] args) throws IOException {
        parseOptions(args);
        IndiceeAPIConnector connector = new IndiceeAPIConnector(host, consumerKey, consumerSecret, tokenKey,
                tokenSecret);
        ContributionService contributionService = new ContributionServiceImpl(connector);

        if (contributionDirectory != null) {
            File datasets[] = contributionDirectory.listFiles();
            for (int i = 0; i < datasets.length; i++) {
                try {
                    contributionService.upload(datasets[i]);
                } catch (IndiceeAPIConnectorException e) {
                    System.out.println(i + ") failed: " + datasets[i].getName());
                    System.exit(1);
                }
                System.out.println(i + ") success: " + datasets[i].getName());
            }
        } else if (contributionFile != null) {
            try {
                contributionService.upload(contributionFile);
            } catch (IndiceeAPIConnectorException e) {
                System.out.println("Failed to upload: " + contributionFile.getName());
                System.exit(1);
            }
            System.out.println("Successfully uploaded: " + contributionFile.getName());
        }
    }

    /**
     * Parses the arguements
     * 
     * @param args
     */
    @SuppressWarnings("static-access")
    private static void parseOptions(String[] args) {
        Options options = new Options();
        options.addOption(OptionBuilder.withArgName("url").hasArg().withDescription(
                "The Indicee server to connect to. e.g. https://secure.indicee.com").isRequired().withLongOpt("url")
                .create(URL_OPTION));
        options.addOption(OptionBuilder.withArgName("consumerKey").hasArg().withDescription("The consumer key to use.")
                .isRequired().withLongOpt("consumer-key").create(CONSUMER_KEY_OPTION));
        options.addOption(OptionBuilder.withArgName("consumerSecret").hasArg().withDescription(
                "The consumer secret to use.").isRequired().withLongOpt("consumer-secret").create(
                CONSUMER_SECRET_OPTION));
        options.addOption(OptionBuilder.withArgName("tokenKey").hasArg().withDescription("The token key to use.")
                .isRequired().withLongOpt("token-key").create(TOKEN_KEY_OPTION));
        options.addOption(OptionBuilder.withArgName("tokenSecret").hasArg().withDescription("The token secret to use.")
                .isRequired().withLongOpt("token-secret").create(TOKEN_SECRET_OPTION));
        OptionGroup optionGroup = new OptionGroup();
        optionGroup.addOption(OptionBuilder.withArgName("uploadDirectory").hasArg().withDescription(
                "The directory containing all the files to upload.").isRequired().withLongOpt("directory").create(
                DIRECTORY_OPTION));
        optionGroup.addOption(OptionBuilder.withArgName("uploadFile").hasArg().withDescription("The file to upload.")
                .isRequired().withLongOpt("file").create(FILE_OPTION));
        optionGroup.setRequired(true);
        options.addOptionGroup(optionGroup);
        options.addOption(HELP_OPTION, "help", false, "Prints out this usage information.");

        CommandLineParser parser = new PosixParser();
        CommandLine cmd = null;
        try {
            cmd = parser.parse(options, args);

        } catch (ParseException e) {
            // oops, something went wrong
            System.err.println("Parsing failed.  Reason: " + e.getMessage());
            printHelp(options);
            System.exit(1);
        }

        if (cmd.hasOption(HELP_OPTION)) {
            printHelp(options);
            System.exit(0);
        }
        if (cmd.hasOption(URL_OPTION)) {
            host = cmd.getOptionValue(URL_OPTION);
        }
        if (cmd.hasOption(CONSUMER_KEY_OPTION)) {
            consumerKey = cmd.getOptionValue(CONSUMER_KEY_OPTION);
        }
        if (cmd.hasOption(CONSUMER_SECRET_OPTION)) {
            consumerSecret = cmd.getOptionValue(CONSUMER_SECRET_OPTION);
        }
        if (cmd.hasOption(TOKEN_KEY_OPTION)) {
            tokenKey = cmd.getOptionValue(TOKEN_KEY_OPTION);
        }
        if (cmd.hasOption(TOKEN_SECRET_OPTION)) {
            tokenSecret = cmd.getOptionValue(TOKEN_SECRET_OPTION);
        }

        if (cmd.hasOption(DIRECTORY_OPTION)) {
            contributionDirectory = (new File(cmd.getOptionValue(DIRECTORY_OPTION)));
            // validate the directory
            if (!contributionDirectory.exists()) {
                System.err.println("Error: The contribution directory does not exist: "
                        + cmd.getOptionValue(DIRECTORY_OPTION));
                System.exit(1);
            } else if (!contributionDirectory.isDirectory()) {
                System.err.println("Error: The contribution directory is not a directory: "
                        + cmd.getOptionValue(DIRECTORY_OPTION));
                System.exit(1);
            } else if (!contributionDirectory.canRead()) {
                System.err.println("Error: The contribution directory cannot be read: "
                        + cmd.getOptionValue(DIRECTORY_OPTION));
                System.exit(1);
            }
        }
        if (cmd.hasOption(FILE_OPTION)) {
            contributionFile = (new File(cmd.getOptionValue(FILE_OPTION)));
            // validate the file
            if (!contributionFile.exists()) {
                System.err.println("Error: The contribution file does not exist: " + cmd.getOptionValue(FILE_OPTION));
                System.exit(1);
            } else if (contributionFile.isDirectory()) {
                System.err.println("Error: The contribution file is a directory: " + cmd.getOptionValue(FILE_OPTION));
                System.exit(1);
            } else if (!contributionFile.canRead()) {
                System.err.println("Error: The contribution file cannot be read: " + cmd.getOptionValue(FILE_OPTION));
                System.exit(1);
            }
        }
    }

    /**
     * Prints the help
     * 
     * @param options
     */
    private static void printHelp(Options options) {
        HelpFormatter formatter = new HelpFormatter();
        formatter
                .printHelp(
                        "FileUploader -u <url> -ck <consumerKey> -cs <consumerSecret> -tk <tokenKey> -ts <tokenSecret> [-d <uploadDirectory> | -f <uploadFile>]",
                        options);
    }
}
